@@ -20,8 +20,6 @@ class Report(models.Model):
     modified = models.DateField(auto_now=True)
     distinct = models.BooleanField()
     
-    
-    
     @models.permalink
     def get_absolute_url(self):
         return ("report_update_view", [str(self.id)])
@@ -42,7 +40,8 @@ class DisplayField(models.Model):
     aggregate = models.CharField(
         max_length=5,
         choices = (
-            ('Count','Sum'),
+            ('Sum','Sum'),
+            ('Count','Count'),
             ('Ave','Ave'),
             ('Max','Max'),
             ('Min','Min'),
@@ -96,4 +95,4 @@ class FilterField(models.Model):
         ordering = ['position']
     def __unicode__(self):
         return self.field
-    
\ No newline at end of file
+    
