@@ -2,7 +2,7 @@ from django.contrib.contenttypes.models import ContentType
 from django.core import exceptions
 from django.contrib.admin.views.decorators import staff_member_required
 from django.contrib.auth.decorators import permission_required
-from django.db.models import Avg, Max, Min, Count
+from django.db.models import Avg, Max, Min, Count, Sum, CharField
 from django.forms.models import inlineformset_factory
 from django.http import HttpResponse, HttpResponseRedirect, HttpResponseForbidden
 from django.shortcuts import render_to_response, get_object_or_404
@@ -11,12 +11,16 @@ from report_builder.models import Report, DisplayField, FilterField
 from django.utils.decorators import method_decorator
 from django.views.generic.edit import CreateView
 from django.views.generic.edit import UpdateView
-
 from django import forms
 
 import datetime
+import time
+import re
+from decimal import Decimal
+
 from dateutil import parser
 
+
 class ReportForm(forms.ModelForm):
     class Meta:
         model = Report
@@ -78,6 +82,70 @@ def get_direct_fields_from_model(model_class):
         if field[2] and not field[3] and field[0].__class__.__name__ != "ForeignKey":
             direct_fields += [field[0]]
     return direct_fields
+
+def get_properties_from_model(model_class):
+    properties = []
+    for attr_name, attr in dict(model_class.__dict__).iteritems():
+        if type(attr) == property:
+            properties.append(attr_name)
+    return sorted(properties)
+
+def filter_property(objects_list, filter_field, value):
+    filter_type = filter_field.filter_type
+    filter_value = filter_field.filter_value
+    #TODO: i10n
+    WEEKDAY_INTS = {
+        'monday': 0,
+        'tuesday': 1,
+        'wednesday': 2,
+        'thursday': 3,
+        'friday': 4,
+        'saturday': 5,
+        'sunday': 6,
+    }
+    if filter_type == 'exact' and str(value) == filter_value:
+        return False
+    if filter_type == 'iexact' and str(value).lower() == str(filter_value).lower():
+        return False
+    if filter_type == 'contains' and filter_value in value:
+        return False
+    if filter_type == 'icontains' and str(filter_value).lower() in str(value).lower():
+        return False
+    if filter_type == 'in' and value in filter_value:
+        return False
+    # convert dates and datetimes to timestamps in order to compare digits and date/times the same
+    if isinstance(value, datetime.datetime) or isinstance(value, datetime.date): 
+        value = str(time.mktime(value.timetuple())) 
+        filter_value_dt = parser.parse(filter_value)
+        filter_value = str(time.mktime(filter_value_dt.timetuple()))
+    if filter_type == 'gt' and Decimal(value) > Decimal(filter_value):
+        return False
+    if filter_type == 'gte' and Decimal(value) >= Decimal(filter_value):
+        return False
+    if filter_type == 'lt' and Decimal(value) < Decimal(filter_value):
+        return False
+    if filter_type == 'lte' and Decimal(value) <= Decimal(filter_value):
+        return False
+    if filter_type == 'startswith' and str(value).startswith(str(filter_value)):
+        return False
+    if filter_type == 'istartswith' and str(value).lower().startswith(str(filter_value)):
+        return False
+    if filter_type == 'endswith' and str(value).endswith(str(filter_value)):
+        return False
+    if filter_type == 'iendswith' and str(value).lower().endswith(str(filter_value)):
+        return False
+    if filter_type == 'range' and value in [int(x) for x in filter_value]:
+        return False
+    if filter_type == 'week_day' and WEEKDAY_INTS.get(str(filter_value).lower()) == value.weekday:
+        return False
+    if filter_type == 'isnull' and value == None:
+        return False
+    if filter_type == 'regex' and re.search(filter_value, value):
+        return False
+    if filter_type == 'iregex' and re.search(filter_value, value, re.I):
+        return False
+    return True
+
             
 def ajax_get_related(request):
     """ Get related model and fields
@@ -99,7 +167,7 @@ def ajax_get_related(request):
     
     new_fields = get_relation_fields_from_model(new_model)
     model_ct = ContentType.objects.get_for_model(new_model)
-    
+
     if path_verbose:
         path_verbose += "::"
     path_verbose += field[0].name
@@ -115,16 +183,19 @@ def ajax_get_related(request):
     }, RequestContext(request, {}),)
 
 def ajax_get_fields(request):
-    """ Get fields for a particular model
+    """ Get fields and properties for a particular model
     """
-    field_name = request.GET['field']
+    field_name = request.GET.get('field')
     model = ContentType.objects.get(pk=request.GET['model']).model_class()
     path = request.GET['path']
-    path_verbose = request.GET['path_verbose']
-    
+    path_verbose = request.GET.get('path_verbose')
+    properties = get_properties_from_model(model)
+
     if field_name == '':
         return render_to_response('report_builder/report_form_fields_li.html', {
             'fields': get_direct_fields_from_model(model),
+            'properties': properties,
+
         }, RequestContext(request, {}),)
     
     field = model._meta.get_field_by_name(field_name)
@@ -137,15 +208,17 @@ def ajax_get_fields(request):
     
     if field[2]:
         # Direct field
-        new_model = field[0].related.parent_model()
+        new_model = field[0].related.parent_model
     else:
         # Indirect related field
-        new_model = field[0].model()
-    
+        new_model = field[0].model
+   
     fields = get_direct_fields_from_model(new_model)
+    properties = get_properties_from_model(new_model)
     
     return render_to_response('report_builder/report_form_fields_li.html', {
         'fields': fields,
+        'properties': properties,
         'path': path,
         'path_verbose': path_verbose,
     }, RequestContext(request, {}),)
@@ -177,6 +250,10 @@ def report_to_list(report, user, preview=False):
     # Filters
     for filter_field in report.filterfield_set.all():
         try:
+            # exclude properties from standard ORM filtering 
+            if '[property]' in filter_field.field_verbose:
+                continue
+
             filter_string = str(filter_field.path + filter_field.field)
             
             if filter_field.filter_type:
@@ -192,12 +269,13 @@ def report_to_list(report, user, preview=False):
                 else:
                     filter_value = filter_field.filter_value
                 filter_list = {filter_string: filter_value}
-                
+            
             if not filter_field.exclude:
                 objects = objects.filter(**filter_list)
             else:
                 objects = objects.exclude(**filter_list)
-        except:
+
+        except Exception, e:
             message += "Filter Error on %s. If you are using the report builder then " % filter_field.field_verbose
             message += "you found a bug! "
             message += "If you made this in admin, then you probably did something wrong."
@@ -212,7 +290,9 @@ def report_to_list(report, user, preview=False):
             objects = objects.annotate(Min(display_field.path + display_field.field))
         elif display_field.aggregate == "Count":
             objects = objects.annotate(Count(display_field.path + display_field.field))
-    
+        elif display_field.aggregate == "Sum":
+            objects = objects.annotate(Sum(display_field.path + display_field.field))
+
     # Ordering
     order_list = []
     for display_field in report.displayfield_set.filter(sort__isnull=False).order_by('sort'):
@@ -232,12 +312,15 @@ def report_to_list(report, user, preview=False):
     
     # Display Values
     values_list = []
-    for display_field in report.displayfield_set.all():
+    property_list = {} 
+    for i, display_field in enumerate(report.displayfield_set.all()):
         model = get_model_from_path_string(model_class, display_field.path)
         if user.has_perm(model._meta.app_label + '.change_' + model._meta.module_name) \
         or user.has_perm(model._meta.app_label + '.view_' + model._meta.module_name) \
         or not model:
-            if display_field.aggregate == "Avg":
+            if '[property]' in display_field.field_verbose:
+                property_list[i] = display_field.path + display_field.field
+            elif display_field.aggregate == "Avg":
                 values_list += [display_field.path + display_field.field + '__ave']
             elif display_field.aggregate == "Max":
                 values_list += [display_field.path + display_field.field + '__max']
@@ -245,6 +328,8 @@ def report_to_list(report, user, preview=False):
                 values_list += [display_field.path + display_field.field + '__min']
             elif display_field.aggregate == "Count":
                 values_list += [display_field.path + display_field.field + '__count']
+            elif display_field.aggregate == "Sum":
+                values_list += [display_field.path + display_field.field + '__sum']
             else:
                 values_list += [display_field.path + display_field.field]
         else:
@@ -252,7 +337,34 @@ def report_to_list(report, user, preview=False):
     try:
         if user.has_perm(report.root_model.app_label + '.change_' + report.root_model.model) \
         or user.has_perm(report.root_model.app_label + '.view_' + report.root_model.model):
-            objects_list = objects.values_list(*values_list)
+            objects_list = []
+            # need to get values_list in order to traverse relations and get aggregates
+            # need objects for properties
+
+            property_filters = {} 
+            for property_filter in report.filterfield_set.filter(field_verbose__contains='[property]'):
+                property_filters[property_filter.field] = property_filter 
+
+            objects_list = list(objects.values_list(*values_list))
+            objects = list(objects)
+
+            if property_list: 
+                filtered_objects_list = []
+                for i, obj in enumerate(objects):
+                    remove_row = False
+                    for position, display_property in property_list.iteritems(): 
+                        val = reduce(getattr, display_property.split('__'), obj)
+                        # change tuples to lists in order to insert
+                        objects_list[i] = list(objects_list[i])
+                        objects_list[i].insert(position, val)
+                        # TODO: move property filter so you don't have to display properties to filter
+                        pf = property_filters.get(display_property)
+                        if pf and filter_property(objects_list, pf, val):
+                            remove_row = True
+                            break
+                    if not remove_row:
+                        filtered_objects_list += [objects_list[i]]
+                objects_list = filtered_objects_list
         else:
             objects_list = []
             message = "Permission Denied on %s" % report.root_model.name
@@ -260,6 +372,7 @@ def report_to_list(report, user, preview=False):
         message += "Field Error. If you are using the report builder then you found a bug!"
         message += "If you made this in admin, then you probably did something wrong."
         objects_list = None
+
     
     return objects_list, message
     
@@ -289,11 +402,12 @@ class ReportUpdateView(UpdateView):
     @method_decorator(permission_required('report_builder.change_report'))
     def dispatch(self, request, *args, **kwargs):
         return super(ReportUpdateView, self).dispatch(request, *args, **kwargs)
-    
+
     def get_context_data(self, **kwargs):
         ctx = super(ReportUpdateView, self).get_context_data(**kwargs)
         model_class = self.object.root_model.model_class()
         model_ct = ContentType.objects.get_for_model(model_class)
+        properties = get_properties_from_model(model_class)
         
         direct_fields = get_direct_fields_from_model(model_class)
         relation_fields = get_relation_fields_from_model(model_class)
@@ -321,10 +435,11 @@ class ReportUpdateView(UpdateView):
         
         ctx['related_fields'] = relation_fields
         ctx['fields'] = direct_fields
+        ctx['properties'] = properties
         ctx['model_ct'] = model_ct
         
         return ctx
-    
+
     def form_valid(self, form):
         context = self.get_context_data()
         field_list_formset = context['field_list_formset']
@@ -379,4 +494,4 @@ def download_xlsx(request, pk):
     return response
     
     
-    
\ No newline at end of file
+    
